@@ -1,17 +1,19 @@
 
+from __future__ import absolute_import
 import sys, os, stat, time, random, subprocess, glob
 
 from wvtest import *
 
-from bup import client, git
+from bup import client, git, path
+from bup.compat import bytes_from_uint, environ, range
 from bup.helpers import mkdirp
 from buptest import no_lingering_errors, test_tempdir
 
 
 def randbytes(sz):
-    s = ''
-    for i in xrange(sz):
-        s += chr(random.randrange(0,256))
+    s = b''
+    for i in range(sz):
+        s += bytes_from_uint(random.randrange(0,256))
     return s
 
 
@@ -19,15 +21,14 @@ s1 = randbytes(10000)
 s2 = randbytes(10000)
 s3 = randbytes(10000)
 
-IDX_PAT = '/*.idx'
+IDX_PAT = b'/*.idx'
     
 
 @wvtest
 def test_server_split_with_indexes():
     with no_lingering_errors():
-        with test_tempdir('bup-tclient-') as tmpdir:
-            os.environ['BUP_MAIN_EXE'] = '../../../bup'
-            os.environ['BUP_DIR'] = bupdir = tmpdir
+        with test_tempdir(b'bup-tclient-') as tmpdir:
+            environ[b'BUP_DIR'] = bupdir = tmpdir
             git.init_repo(bupdir)
             lw = git.PackWriter()
             c = client.Client(bupdir, create=True)
@@ -45,9 +46,8 @@ def test_server_split_with_indexes():
 @wvtest
 def test_multiple_suggestions():
     with no_lingering_errors():
-        with test_tempdir('bup-tclient-') as tmpdir:
-            os.environ['BUP_MAIN_EXE'] = '../../../bup'
-            os.environ['BUP_DIR'] = bupdir = tmpdir
+        with test_tempdir(b'bup-tclient-') as tmpdir:
+            environ[b'BUP_DIR'] = bupdir = tmpdir
             git.init_repo(bupdir)
 
             lw = git.PackWriter()
@@ -56,7 +56,7 @@ def test_multiple_suggestions():
             lw = git.PackWriter()
             lw.new_blob(s2)
             lw.close()
-            WVPASSEQ(len(glob.glob(git.repo('objects/pack'+IDX_PAT))), 2)
+            WVPASSEQ(len(glob.glob(git.repo(b'objects/pack'+IDX_PAT))), 2)
 
             c = client.Client(bupdir, create=True)
             WVPASSEQ(len(glob.glob(c.cachedir+IDX_PAT)), 0)
@@ -64,11 +64,39 @@ def test_multiple_suggestions():
             s1sha = rw.new_blob(s1)
             WVPASS(rw.exists(s1sha))
             s2sha = rw.new_blob(s2)
+
             # This is a little hacky, but ensures that we test the
-            # code under test
+            # code under test. First, flush to ensure that we've
+            # actually sent all the command ('receive-objects-v2')
+            # and their data to the server. This may be needed if
+            # the output buffer size is bigger than the data (both
+            # command and objects) we're writing. To see the need
+            # for this, change the object sizes at the beginning
+            # of this file to be very small (e.g. 10 instead of 10k)
+            c.conn.outp.flush()
+
+            # Then, check if we've already received the idx files.
+            # This may happen if we're preempted just after writing
+            # the data, then the server runs and suggests, and only
+            # then we continue in PackWriter_Remote::_raw_write()
+            # and check the has_input(), in that case we'll receive
+            # the idx still in the rw.new_blob() calls above.
+            #
+            # In most cases though, that doesn't happen, and we'll
+            # get past the has_input() check before the server has
+            # a chance to respond - it has to actually hash the new
+            # object here, so it takes some time. So also break out
+            # of the loop if the server has sent something on the
+            # connection.
+            #
+            # Finally, abort this after a little while (about one
+            # second) just in case something's actually broken.
+            n = 0
             while (len(glob.glob(c.cachedir+IDX_PAT)) < 2 and
-                   not c.conn.has_input()):
-                pass
+                   not c.conn.has_input() and n < 10):
+                time.sleep(0.1)
+                n += 1
+            WVPASS(len(glob.glob(c.cachedir+IDX_PAT)) == 2 or c.conn.has_input())
             rw.new_blob(s2)
             WVPASS(rw.objcache.exists(s1sha))
             WVPASS(rw.objcache.exists(s2sha))
@@ -81,11 +109,10 @@ def test_multiple_suggestions():
 @wvtest
 def test_dumb_client_server():
     with no_lingering_errors():
-        with test_tempdir('bup-tclient-') as tmpdir:
-            os.environ['BUP_MAIN_EXE'] = '../../../bup'
-            os.environ['BUP_DIR'] = bupdir = tmpdir
+        with test_tempdir(b'bup-tclient-') as tmpdir:
+            environ[b'BUP_DIR'] = bupdir = tmpdir
             git.init_repo(bupdir)
-            open(git.repo('bup-dumb-server'), 'w').close()
+            open(git.repo(b'bup-dumb-server'), 'w').close()
 
             lw = git.PackWriter()
             lw.new_blob(s1)
@@ -104,9 +131,8 @@ def test_dumb_client_server():
 @wvtest
 def test_midx_refreshing():
     with no_lingering_errors():
-        with test_tempdir('bup-tclient-') as tmpdir:
-            os.environ['BUP_MAIN_EXE'] = bupmain = '../../../bup'
-            os.environ['BUP_DIR'] = bupdir = tmpdir
+        with test_tempdir(b'bup-tclient-') as tmpdir:
+            environ[b'BUP_DIR'] = bupdir = tmpdir
             git.init_repo(bupdir)
             c = client.Client(bupdir, create=True)
             rw = c.new_packwriter()
@@ -119,7 +145,7 @@ def test_midx_refreshing():
             p2name = os.path.join(c.cachedir, p2base)
             del rw
 
-            pi = git.PackIdxList(bupdir + '/objects/pack')
+            pi = git.PackIdxList(bupdir + b'/objects/pack')
             WVPASSEQ(len(pi.packs), 2)
             pi.refresh()
             WVPASSEQ(len(pi.packs), 2)
@@ -132,7 +158,7 @@ def test_midx_refreshing():
             WVFAIL(p2.exists(s1sha))
             WVPASS(p2.exists(s2sha))
 
-            subprocess.call([bupmain, 'midx', '-f'])
+            subprocess.call([path.exe(), b'midx', b'-f'])
             pi.refresh()
             WVPASSEQ(len(pi.packs), 1)
             pi.refresh(skip_midx=True)
@@ -145,18 +171,18 @@ def test_midx_refreshing():
 def test_remote_parsing():
     with no_lingering_errors():
         tests = (
-            (':/bup', ('file', None, None, '/bup')),
-            ('file:///bup', ('file', None, None, '/bup')),
-            ('192.168.1.1:/bup', ('ssh', '192.168.1.1', None, '/bup')),
-            ('ssh://192.168.1.1:2222/bup', ('ssh', '192.168.1.1', '2222', '/bup')),
-            ('ssh://[ff:fe::1]:2222/bup', ('ssh', 'ff:fe::1', '2222', '/bup')),
-            ('bup://foo.com:1950', ('bup', 'foo.com', '1950', None)),
-            ('bup://foo.com:1950/bup', ('bup', 'foo.com', '1950', '/bup')),
-            ('bup://[ff:fe::1]/bup', ('bup', 'ff:fe::1', None, '/bup')),)
+            (b':/bup', (b'file', None, None, b'/bup')),
+            (b'file:///bup', (b'file', None, None, b'/bup')),
+            (b'192.168.1.1:/bup', (b'ssh', b'192.168.1.1', None, b'/bup')),
+            (b'ssh://192.168.1.1:2222/bup', (b'ssh', b'192.168.1.1', b'2222', b'/bup')),
+            (b'ssh://[ff:fe::1]:2222/bup', (b'ssh', b'ff:fe::1', b'2222', b'/bup')),
+            (b'bup://foo.com:1950', (b'bup', b'foo.com', b'1950', None)),
+            (b'bup://foo.com:1950/bup', (b'bup', b'foo.com', b'1950', b'/bup')),
+            (b'bup://[ff:fe::1]/bup', (b'bup', b'ff:fe::1', None, b'/bup')),)
         for remote, values in tests:
             WVPASSEQ(client.parse_remote(remote), values)
         try:
-            client.parse_remote('http://asdf.com/bup')
+            client.parse_remote(b'http://asdf.com/bup')
             WVFAIL()
         except client.ClientError:
             WVPASS()
