@@ -2,10 +2,8 @@
 
 bup_t_lib_script_home=$(cd "$(dirname $0)" && pwd) || exit $?
 
-bup-python()
-{
-    "$bup_t_lib_script_home/../cmd/bup-python" "$@"
-}
+bup-cfg-py() { "$bup_t_lib_script_home/../config/bin/python" "$@"; }
+bup-python() { "$bup_t_lib_script_home/../dev/bup-python" "$@"; }
 
 force-delete()
 {
@@ -17,7 +15,7 @@ resolve-parent()
     test "$#" -eq 1 || return $?
     echo "$1" | \
         PYTHONPATH="$bup_t_lib_script_home/../lib" bup-python -c \
-        "import sys, bup.helpers; print bup.helpers.resolve_parent(sys.stdin.readline())" \
+        "import sys, bup.helpers; print(bup.helpers.resolve_parent(sys.stdin.readline()))" \
         || return $?
 }
 
