@@ -7,6 +7,7 @@ set -o pipefail
 root_status="$(t/root-status)" || exit $?
 
 TOP="$(WVPASS pwd)" || exit $?
+export PATH="$TOP/t/bin:$PATH"
 tmpdir="$(WVPASS wvmktempdir)" || exit $?
 export BUP_DIR="$tmpdir/bup"
 
@@ -43,10 +44,11 @@ id-other-than()
 genstat()
 {
     (
-        export PATH="$TOP:$PATH" # pick up bup
+        export PATH="$TOP/bin:$PATH" # pick up bup
+        bup version
         # Skip atime (test elsewhere) to avoid the observer effect.
-        WVPASS find . | WVPASS sort \
-            | WVPASS xargs bup xstat \
+        WVPASS find . -print0 | WVPASS sort-z \
+            | WVPASS xargs -0 bup xstat \
             --mtime-resolution "$mtime_resolution"ns \
             --exclude-fields ctime,atime,size
     )
@@ -438,10 +440,11 @@ src/foo/3"
 
     tmpdir="$(WVPASS wvmktempdir)" || exit $?    
 
-    first_group="$(WVPASS bup-python -c 'import os,grp; \
-      print grp.getgrgid(os.getgroups()[0])[0]')" || exit $?
-    last_group="$(bup-python -c 'import os,grp; \
-      print grp.getgrgid(os.getgroups()[-1])[0]')" || exit $?
+    # FIXME: binary groups
+    first_group="$(WVPASS bup-cfg-py -c 'import os,grp; \
+      print(grp.getgrgid(os.getgroups()[0])[0])')" || exit $?
+    last_group="$(bup-cfg-py -c 'import os,grp; \
+      print(grp.getgrgid(os.getgroups()[-1])[0])')" || exit $?
     last_group_erx="$(escape-erx "$last_group")"
 
     WVSTART 'metadata (restoration of ownership)'
@@ -677,11 +680,10 @@ if [ "$root_status" = root ]; then
         (
             WVPASS mkdir "$testfs"/src/foo
             WVPASS touch "$testfs"/src/bar
-            PYTHONPATH="$TOP/lib" \
-                WVPASS bup-python -c "from bup import xstat; \
+            WVPASS bup-python -c "from bup import xstat; \
                 x = xstat.timespec_to_nsecs((42, 0));\
-                   xstat.utime('$testfs/src/foo', (x, x));\
-                   xstat.utime('$testfs/src/bar', (x, x));"
+                xstat.utime(b'$testfs/src/foo', (x, x));\
+                xstat.utime(b'$testfs/src/bar', (x, x));"
             WVPASS cd "$testfs"
             WVPASS bup meta -v --create --recurse --file src.meta src
             WVPASS bup meta -tvf src.meta
@@ -707,7 +709,7 @@ if [ "$root_status" = root ]; then
         (
             WVPASS touch "$testfs"/src/foo
             WVPASS mkdir "$testfs"/src/bar
-            WVPASS chattr +acdeijstuADST "$testfs"/src/foo
+            WVPASS chattr +acdeijstuADS "$testfs"/src/foo
             WVPASS chattr +acdeijstuADST "$testfs"/src/bar
             (WVPASS cd "$testfs"; WVPASS test-src-create-extract) || exit $?
             # Test restoration to a limited filesystem (vfat).
@@ -719,7 +721,7 @@ if [ "$root_status" = root ]; then
                 WVPASS cd "$testfs_limited"/src-restore
                 WVFAIL bup meta --extract --file "$testfs"/src.meta 2>&1 \
                     | WVPASS grep -e '^Linux chattr:' \
-                    | WVPASS bup-python -c \
+                    | WVPASS bup-cfg-py -c \
                     'import sys; exit(not len(sys.stdin.readlines()) == 3)'
             ) || exit $?
         ) || exit $?
@@ -742,8 +744,8 @@ if [ "$root_status" = root ]; then
             WVPASS cd "$testfs_limited"/src-restore
             WVFAIL bup meta --extract --file "$testfs"/src.meta
             WVFAIL bup meta --extract --file "$testfs"/src.meta 2>&1 \
-                | WVPASS grep -e "^xattr\.set '" \
-                | WVPASS bup-python -c \
+                | WVPASS grep -e "^xattr\.set u\?'" \
+                | WVPASS bup-cfg-py -c \
                 'import sys; exit(not len(sys.stdin.readlines()) == 2)'
         ) || exit $?
 
@@ -765,7 +767,7 @@ if [ "$root_status" = root ]; then
             WVPASS cd "$testfs_limited"/src-restore
             WVFAIL bup meta --extract --file "$testfs"/src.meta 2>&1 \
                 | WVPASS grep -e '^POSIX1e ACL applyto:' \
-                | WVPASS bup-python -c \
+                | WVPASS bup-cfg-py -c \
                 'import sys; exit(not len(sys.stdin.readlines()) == 2)'
         ) || exit $?
 
