@@ -43,4 +43,16 @@ WVPASS git cat-file commit "$commit_id" | head -n 1 \
 WVPASS bup join baz > restore-baz
 WVPASS cmp src/baz restore-baz
 
+WVSTART "index-cache"
+# the 'a-zA-Z0-9_' is '\w' from python,
+# the trailing _ is because there's no dir specified
+# and that should thus be empty
+hostname=$(uname -n)
+idxcache=$(echo "$hostname" | sed 's/[^@a-zA-Z0-9_]/_/g')_
+# there should be an index-cache now
+for idx in "$tmpdir"/bup/objects/pack/*.idx ; do
+    cachedidx="$tmpdir/bup/index-cache/$idxcache/$(basename "$idx")"
+    WVPASS cmp "$idx" "$cachedidx"
+done
+
 WVPASS rm -rf "$tmpdir"
